unit RS232_Handle;

interface

type TPBA= word;
     TLink= (lnkCTS, lnkDSR, lnkRI, lnkDCD, lnkDTR, lnkRTS, lnkTxD);

const
     {port address}
     COM1= $3F8;
     COM2= $2F8;
     COM3= $3E8;
     COM4= $3F8;
     {links allocations}
     alCTS= $10;
     alDSR= $20;
     alRI = $40;
     alDCD= $80;
     alDTR= $01;
     alRTS= $02;
     alTxD= $40;
     {RxD can't be used}
     {Links' Registers' offsets}
     ofsCTS= 6;
     ofsDSR= 6;
     ofsRI = 6;
     ofsDCD= 6;
     {*}
     ofsDTR= 4;
     ofsRTS= 4;
     {*}
     ofsTxD= 3;

var  PBA: TPBA;

{ General Port Handle }
function ReadPort(pAddress: TPBA):byte;
procedure WritePort(pAddress: TPBA; Data: byte);

{ SCAN Inputs }
function Scan_CTS(PA: TPBA): boolean;
function Scan_DSR(PA: TPBA): boolean;
function Scan_RI (PA: TPBA): boolean;
function Scan_DCD(PA: TPBA): boolean;

{ SCAN Outputs }
function Scan_DTR(PA: TPBA): boolean;
function Scan_RTS(PA: TPBA): boolean;
function Scan_TxD(PA: TPBA): boolean;

{ SCAN User }
function ScanUserLink(Link: TLink): boolean;

{ Direct Handling }
procedure Write_TxD(value: boolean);
procedure Write_RTS(value: boolean);
procedure Write_DTR(value: boolean);

{ Links Tabs Allocations }
function GetLinkOffset(Link: TLink): byte;
function GetLinkAlloc(Link: TLink) : byte;

implementation

{ General Port Communications Routines }
function ReadPort(pAddress: Word):byte;
var InReadPort: byte;
begin
  asm
    Mov dx, pAddress
    In  al, dx
    Nop
    Nop
    Mov byte ptr InReadPort, al
  end;
  ReadPort:= InReadPort;
end;

procedure WritePort(pAddress: Word; Data: byte);
begin
  asm
    Mov dx, pAddress
    Mov al, Data
    Out dx, al
  end;
end;

{ Inputs Scan Routines }

function Scan_CTS(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $10) = $10;
end;

function Scan_DSR(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $20) = $20;
end;

function Scan_RI (PA: TPBA): boolean;
begin
  result:=  (ReadPort(PA+6) and $40) = $40;
end;

function Scan_DCD(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $80) = $80;
end;

{ Outputs Scan Routines }

function Scan_DTR(PA: TPBA): boolean;
begin
  Scan_DTR:= (ReadPort(PA+4) and $01) = $01;
end;

function Scan_RTS(PA: TPBA): boolean;
begin
  Scan_RTS:= (ReadPort(PA+4) and $02) = $02;
end;

function Scan_TxD(PA: TPBA): boolean;
begin
  Scan_TxD:= (ReadPort(PA+3) and $40) = $40;
end;

{ Scan User }
function GetLinkOffset(Link: TLink): byte;
begin
  case Link of
    lnkCTS..lnkDCD: result:= 6;
    lnkDTR..lnkRTS: result:= 4;
    lnkTxD     : result:= 3;
  end;
end;

{ Tabs }
function GetLinkAlloc(Link: TLink) : byte;
begin
  case Link of
    lnkCTS: result:= alCTS;
    lnkDSR: result:= alDSR;
    lnkRI : result:= alRI;
    lnkDCD: result:= alDCD;
    lnkDTR: result:= alDTR;
    lnkRTS: result:= alRTS;
    lnkTxD: result:= alTxD;
  end;
end;

function ScanUserLink(Link: TLink): boolean;
begin
  ScanUserLink:= ReadPort(PBA + GetLinkOffset(Link)) and GetLinkAlloc(Link) = GetLinkAlloc(Link);
end;

procedure Write_TxD(value: boolean);
begin
  case value of
    true: WritePort(PBA + 3, ReadPort(PBA + 3) or alTxD);
    false:WritePort(PBA + 3, ReadPort(PBA + 3) and (not alTxD));
  end;
end;

procedure Write_RTS(value: boolean);
begin
  case value of
    true: WritePort(PBA + 4, ReadPort(PBA + 4) or alRTS);
    false:WritePort(PBA + 4, ReadPort(PBA + 4) and (not alRTS));
  end;
end;

procedure Write_DTR(value: boolean);
begin
  case value of
    true: WritePort(PBA + 4, ReadPort(PBA + 4) or alDTR);
    false:WritePort(PBA + 4, ReadPort(PBA + 4) and (not alDTR));
  end;
end;

end.
